/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "NotifyDirCheck.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Work Thread 
//////////////////////////////////////////////////////////////////////

UINT NotifyDirThread( LPVOID pParam )
{
	BOOL             bStop = FALSE;
	HANDLE           hDir  = NULL; 
	CNotifyDirCheck* pNDC  = (CNotifyDirCheck*)pParam;
	FI_List          newFIL,
		             oldFIL;
	EFileAction      faAction;
	CFileInformation fi;

	if( pNDC == NULL )
		return 0;

	hDir = FindFirstChangeNotification( pNDC->GetDirectory(),
										TRUE,
										FILE_NOTIFY_CHANGE_FILE_NAME  |
										FILE_NOTIFY_CHANGE_DIR_NAME   |
										FILE_NOTIFY_CHANGE_SIZE       |
										FILE_NOTIFY_CHANGE_LAST_WRITE |
										FILE_NOTIFY_CHANGE_ATTRIBUTES );

	if( hDir == INVALID_HANDLE_VALUE )
	{
		//ErrorMessage( _T("FindFirstChangeNotification") );
		return 0;
	}
	
 	while( pNDC->IsRun() )
	{
		CFileInformation::RemoveFiles( &oldFIL );
		CFileInformation::EnumFiles( pNDC->GetDirectory(), &oldFIL );

		bStop = FALSE;

		while( WaitForSingleObject( hDir, WAIT_TIMEOUT ) != WAIT_OBJECT_0 )
		{	if( !pNDC->IsRun() )
			{
				bStop = TRUE;//to end
				break;
			}
		}
		if( bStop )
			break;//to end

		CFileInformation::RemoveFiles( &newFIL );
		CFileInformation::EnumFiles( pNDC->GetDirectory(), &newFIL );
		
		Sleep( WAIT_TIMEOUT );

		faAction = CFileInformation::CompareFiles( &oldFIL, &newFIL, fi );

		if( !IS_NOTACT_FILE( faAction ) )
		{
			NOTIFICATION_CALLBACK_PTR ncpAction = pNDC->GetActionCallback();

			if( ncpAction )	//call user's callback
				bStop = ( ncpAction( fi, faAction, pNDC->GetData() ) > 0 );
	
			if( bStop )
				break;//to end
		}
		
		if( FindNextChangeNotification( hDir ) == 0 )
		{
			//ErrorMessage( _T("FindNextChangeNotification") );
			return 0;
		}
	}

	//end point of notification thread
	CFileInformation::RemoveFiles( &newFIL );
	CFileInformation::RemoveFiles( &oldFIL );
	
	return FindCloseChangeNotification( hDir );
}

//////////////////////////////////////////////////////////////////////
// Class 
//////////////////////////////////////////////////////////////////////

CNotifyDirCheck::CNotifyDirCheck()
{
	SetDirectory( "" );
	SetActionCallback( NULL );
	SetData( NULL );
	SetStop();
	m_pThread = NULL;
}

CNotifyDirCheck::~CNotifyDirCheck()
{
	Stop();
}

BOOL CNotifyDirCheck::Run()
{
	if( IsRun() || m_pThread != NULL || m_csDir.IsEmpty() )
		return FALSE;

	SetRun();
	m_pThread = AfxBeginThread( NotifyDirThread, this );
	
	if( m_pThread == NULL )
		SetStop();

	return IsRun();
}

void CNotifyDirCheck::Stop()
{
	if( !IsRun() || m_pThread == NULL )
		return;
	
	SetStop();

	WaitForSingleObject( m_pThread->m_hThread, 2 * NOTIFICATION_TIMEOUT );
	m_pThread = NULL;
}

